@@ -3,6 +3,16 @@ max <- function(..., na.rm = TRUE) suppressWarnings(base::max(..., na.rm = na.rm
 
 arrows <- function(...) suppressWarnings(graphics::arrows(...))
 
+hist.numeric <- function(x, ...) {
+  if(all(!diff(x))) {
+    breaks <- c(0.99, 1.01) * x[1]
+    xlim <- c(0.8, 1.2) * x[1]
+    graphics::hist.default(x, breaks = breaks, xlim = xlim, ...)
+  } else {
+    graphics::hist.default(x, ...)
+  }
+}
+
 #' Get the SAMtool vignettes
 #'
 #' A convenient function to open a web browser with the SAMtool package vignettes
