@@ -13,22 +13,13 @@ RCM_est <- function(x = 1, RCMdata, selectivity, s_selectivity, LWT = list(),
   n_age <- dim(RCMdata@CAA)[2]
   nsurvey <- ncol(RCMdata@Index)
   
-  # Convert to proportions?
+  # Convert to proportions
   RCMdata@CAA <- apply(RCMdata@CAA, c(1, 3), tiny_comp) %>% aperm(c(2, 1, 3))
   RCMdata@CAL <- apply(RCMdata@CAL, c(1, 3), tiny_comp) %>% aperm(c(2, 1, 3))
   
   RCMdata@IAA <- apply(RCMdata@IAA, c(1, 3), tiny_comp) %>% aperm(c(2, 1, 3))
   RCMdata@IAL <- apply(RCMdata@IAL, c(1, 3), tiny_comp) %>% aperm(c(2, 1, 3))
   
-  #Backwards compatibility
-  if(!is.null(dots$ESS) && length(dots$ESS) == 2) {
-    RCMdata@CAA_ESS <- pmin(RCMdata@CAA_ESS, dots$ESS[1])
-    RCMdata@CAL_ESS <- pmin(RCMdata@CAL_ESS, dots$ESS[2])
-    
-    RCMdata@IAA_ESS <- pmin(RCMdata@IAA_ESS, dots$ESS[1])
-    RCMdata@IAL_ESS <- pmin(RCMdata@IAL_ESS, dots$ESS[1])
-  }
-  
   LWT_fleet <- cbind(LWT$Chist, LWT$C_eq, LWT$CAA, LWT$CAL, LWT$MS)
   LWT_index <- cbind(LWT$Index, LWT$IAA, LWT$IAL)
   
